/*********************************************************************
 * Sequence for Desktop Station main
 *
 * Copyright (C) 2015 Yaasan
 *
 */

#include <Arduino.h>
#include <EEPROM.h>
#include <avr/wdt.h>
#include "DSSequence.h"

#define		BTN_NUM5	0b0000000000000001 //SW09
#define		BTN_NUM1	0b0000000000000010 //SW02
#define		BTN_NUM9	0b0000000000000100 //SW13
#define		BTN_NUM4	0b0000000000001000 //SW05
#define		BTN_CLR		0b0000000000010000 //SW14
#define		BTN_NUM0	0b0000000000100000 //SW01
#define		BTN_NUM2	0b0000000001000000 //SW03
#define		BTN_NUM8	0b0000000010000000 //SW12
#define		BTN_ESC		0b0000000100000000 //SW06
#define		BTN_UP		0b0000001000000000 //SW15
#define		BTN_NUM6	0b0000010000000000 //SW06
#define		BTN_NUM3	0b0000100000000000 //SW04
#define		BTN_NUM7	0b0001000000000000 //SW11
#define		BTN_ENT		0b0010000000000000 //SW07
#define		BTN_DN		0b0100000000000000 //SW08

#define		MAX_MAINMENU	5


DSSequence::DSSequence()
{
	
	
	
	gMode = MODE_MAINMENU;
	gNo_CVReadMenu = 0;
	gNo_CVWriteMenu = 0;
	gNo_CVReadWriteMenu = 0;
	gNo_AddrChkMenu = 0;
	
	CVNo = 1;
	CVValue = 0;
	Address = 0;
	
	SignalAddr[0] = 0;
	SignalAddr[1] = 0;
	SignalAddr[2] = 0;
	SignalAddr[3] = 0;
	SignalAddr[4] = 0;
	
	
	
	gDScmd.mCmd = 0;
	
}

PackedDScmd DSSequence::GetCommad()
{
	return gDScmd;
}


void DSSequence::loadEEPROM()
{
	
	if( EEPROM.read(0) == 0xBB)
	{
		
	}	
	
}

void DSSequence::saveEEPROM(byte inMode)
{
	
	if( EEPROM.read(0) == 0xBB)
	{
		
			EEPROM.write(2, lowByte(Address));
			EEPROM.write(3, highByte(Address));
		
		
		
	}
	else
	{
		//初期化
		EEPROM.write(0, 0xBB);
		
		//アドレス書き換え
		EEPROM.write(1, 0);

		
	}
	
}

word DSSequence::getButtonStatus()
{
	
	word aButton_val = 0;
	
	byte aTempBtnA = 0;
	byte aTempBtnB = 0;
	byte aTempBtnC = 0;
	
	unsigned short aButtonA = analogRead(PORT_BTNA);
	unsigned short aButtonB = analogRead(PORT_BTNB);
	unsigned short aButtonC = analogRead(PORT_BTNC);
	
	//Get button status
	aTempBtnA = GetButtonsFromAD(aButtonA);
	aTempBtnB = GetButtonsFromAD(aButtonB);
	aTempBtnC = GetButtonsFromAD(aButtonC);
	
	//Check and process the buttons
	aButton_val = ProcessCheck(aTempBtnA, aTempBtnB, aTempBtnC);
	
	return aButton_val;
	
}







byte DSSequence::Interval()
{
	bool aChange = false;
	word aButton_val = 0;
	
	//Input Buttons
	aButton_val = getButtonStatus();
	
	//Sequence main
	
	switch(gMode)
	{
	case MODE_MAINMENU:
		interval_Main(aButton_val);
		
		break;
		
	case MODE_CVREAD:
		interval_ReadCV(aButton_val);
		
		break;
		
	case MODE_CVWRITE:
		interval_WriteCV(aButton_val);
		
		break;
		
	case MODE_CVREADWRITE:
		interval_ReadWriteCV(aButton_val);
		
		break;
		
	case MODE_NUCKYSIGNAL:
		interval_Signal(aButton_val);
		break;
	case MODE_CHECKADDRESS:
		interval_CheckAddress(aButton_val);
		break;
		
	case MODE_WRITEADDRESS:
		interval_WriteAddress(aButton_val);
		break;		
	}
	
	//Always run event
	interval_always(aButton_val);
	
	
	if( gButtonBuf != aButton_val)
	{
		Serial.print("Button=");
		Serial.println(aButton_val, BIN);
	}	
	
	//Store previous values
	gButtonBuf = aButton_val;
	
	return gDisplayFlag;
}

void DSSequence::interval_always(word inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			
			
			
		}
	}	
	
}

void DSSequence::interval_Main(word inButtonStatus)
{
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			//モード切替
			gMode = gNo_MainMenu + 2;
			
			Serial.print("Next Mode=");
			Serial.println(gMode, DEC);
			
			
			//表示処理
			gDisplayFlag |= 1;
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			if( gNo_MainMenu <= 0)
			{
				gNo_MainMenu = 0;
			}
			else
			{
				gNo_MainMenu--;
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			
			if( gNo_MainMenu >= MAX_MAINMENU)
			{
				gNo_MainMenu = MAX_MAINMENU;
			}
			else
			{
				gNo_MainMenu++;
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}

		

	}	
}

bool DSSequence::editNums(word inButtonStatus, word *iopNum, word inMax)
{
	word aNum = *iopNum;
	bool aRet = false;
	
	if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM1) == true )
	{
		aNum = aNum * 10 + 1;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM2) == true )
	{
		aNum = aNum * 10 + 2;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM3) == true )
	{
		aNum = aNum * 10 + 3;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM4) == true )
	{
		aNum = aNum * 10 + 4;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM5) == true )
	{
		aNum = aNum * 10 + 5;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM6) == true )
	{
		aNum = aNum * 10 + 6;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM7) == true )
	{
		aNum = aNum * 10 + 7;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM8) == true )
	{
		aNum = aNum * 10 + 8;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM9) == true )
	{
		aNum = aNum * 10 + 9;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_NUM0) == true )
	{
		aNum = aNum * 10 + 0;
		aRet = true;
	}
	else if( checkButton(gButtonBuf, inButtonStatus, BTN_CLR) == true )
	{
		aNum = 0;
		aRet = true;
	}	
	
	//Check max value
	if( aNum >= inMax)
	{
		aNum = inMax;
		aRet = true;
	}
	
	
	*iopNum = aNum;
	
	return aRet;
}


void DSSequence::interval_ReadCV(word inButtonStatus)
{
	unsigned char aValue;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_CVReadMenu == 0)
			{
				
				gNo_CVReadMenu = 1;
				
				//読み込み処理実行
				gDScmd.mParam1 = CVNo;
				gDScmd.mCmd = DSCMD_CVREAD;
				gDScmd.mWait = 1;
				
				//実行
				CVExeFunc(&aValue);
				
				//結果表示
				gNo_CVReadMenu = aValue;
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_CVReadMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_CVReadMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			//gNo_CVReadMenu = 0;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			
			//gNo_CVReadMenu = 1;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_CVReadMenu == 0)
			{
				if( editNums(inButtonStatus, &CVNo, 1024) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}

			}
		}		
	}	
}

void DSSequence::interval_WriteCV(word inButtonStatus)
{
	unsigned char aValue;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_CVWriteMenu == 0)
			{
				gNo_CVWriteMenu = 1;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else if( gNo_CVWriteMenu == 1)
			{
				
				gNo_CVWriteMenu = 2;
				
				//読み込み処理実行
				gDScmd.mParam1 = CVNo;
				gDScmd.mParam2 = CVValue;
				gDScmd.mCmd = DSCMD_CVWRITE;
				gDScmd.mWait = 1;
				
				//実行
				CVExeFunc(&aValue);
				
				//結果表示
				gNo_CVWriteMenu = aValue;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_CVWriteMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_CVWriteMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			gNo_CVWriteMenu = 0;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			
			gNo_CVWriteMenu = 1;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_CVWriteMenu == 0)
			{
				if( editNums(inButtonStatus, &CVNo, 1024) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}

			}
			else if( gNo_CVWriteMenu == 1)
			{
				if( editNums(inButtonStatus, &CVValue, 255) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}

			}
		}		
	}	
}

void DSSequence::interval_ReadWriteCV(word inButtonStatus)
{
	unsigned char aValue;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_CVReadWriteMenu == 0)
			{
				gNo_CVReadWriteMenu = 1;
				
				//読み込み処理実行
				gDScmd.mParam1 = CVNo;
				gDScmd.mCmd = DSCMD_CVREAD;
				gDScmd.mWait = 1;
				
				//実行
				CVExeFunc(&aValue);
				
				if( aValue == 2)
				{
					gNo_CVReadWriteMenu = 2;
					
				}
				else
				{
					gNo_CVReadWriteMenu = 6;
					
				}
				
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else if( gNo_CVReadWriteMenu == 2)
			{
				
				gNo_CVReadWriteMenu = 3;
				
				//読み込み処理実行
				gDScmd.mParam1 = CVNo;
				gDScmd.mParam2 = CVValue;
				gDScmd.mCmd = DSCMD_CVWRITE;
				gDScmd.mWait = 1;
				
				//実行
				CVExeFunc(&aValue);
				
				//結果表示
				if( aValue == 3)
				{
					gNo_CVReadWriteMenu = 4;
					
				}
				else
				{
					gNo_CVReadWriteMenu = 5;
					
				}
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_CVReadWriteMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_CVReadWriteMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			gNo_CVReadWriteMenu = 0;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			
			gNo_CVReadWriteMenu = 1;
			
			//表示処理
			//gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_CVReadWriteMenu == 0)
			{
				if( editNums(inButtonStatus, &CVNo, 1023) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}

			}
			else if( gNo_CVReadWriteMenu == 2)
			{
				if( editNums(inButtonStatus, &CVValue, 255) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}

			}
		}		
	}	
}

void DSSequence::interval_Signal(word inButtonStatus)
{

	unsigned char aValue;
	unsigned char aUpperVal;
	unsigned char aLowerVal;
	unsigned char aWriteOk = 0;
	word aTempAddr;
	word aAddr;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_NSignalMenu <= 3)
			{
				gNo_NSignalMenu++;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else if( gNo_NSignalMenu == 4)
			{
				
				gNo_NSignalMenu = 5;
				
				
				for( int i = 0; i < 5; i++)
				{
					
					if( SignalAddr[i] > 0)
					{
						Serial.print("Address");
						Serial.println(i + 1);
						
						//アドレスを-1する（0始まり)
						aAddr = SignalAddr[i] - 1;
						
						//CV値取得
						aTempAddr = ((aAddr) >> 2) + 1;
						aUpperVal = (0b10000000 | (aTempAddr & 0b00111111));//10AAAAAA 
						aLowerVal = (0b10001000 | ((((~aTempAddr) & 0b111000000) >> 2) & 0xFF)) | (((aAddr % 4) & 0b11) << 1);//1AAACDDD
						
						
						//書き込み処理実行
						gDScmd.mParam1 = 13 + (i * 2 + 0);
						gDScmd.mParam2 = aUpperVal;
						gDScmd.mCmd = DSCMD_CVWRITE;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue);
						
						if( aValue == 3)
						{
							aWriteOk++;
						}
						
						//書き込み処理実行
						gDScmd.mParam1 = 13 + (i * 2 + 1);
						gDScmd.mParam2 = aLowerVal;
						gDScmd.mCmd = DSCMD_CVWRITE;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue);
						
						if( aValue == 3)
						{
							aWriteOk++;
						}
						
						Serial.print("CV");
						Serial.print(13 + (i * 2 + 0));
						Serial.print("=");
						Serial.println(aUpperVal);
						
						Serial.print("CV");
						Serial.print(13 + (i * 2 + 1));
						Serial.print("=");
						Serial.println(aLowerVal);
						
					}
					else
					{
						//CVを書き換えない時はOkだったとする。
						aWriteOk += 2;
					}
					
					/* Reset watchdog timer. */
					wdt_reset(); 
				
				}
				
				
				
				if( aWriteOk >= 10)
				{
					//結果表示
					gNo_NSignalMenu = 6;
				}
				else
				{
					//結果表示
					gNo_NSignalMenu = 99;
				}
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_NSignalMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_NSignalMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_UP) == true )
		{
			
			if( gNo_NSignalMenu > 0)
			{
				gNo_NSignalMenu--;
			}
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_DN) == true )
		{
			if( gNo_NSignalMenu <= 3)
			{
				gNo_NSignalMenu++;
			}
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{
			
			if( gNo_NSignalMenu <= 4)
			{
				if( editNums(inButtonStatus, &SignalAddr[gNo_NSignalMenu], 999) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}
			}
		}		
	}	
	
}

void DSSequence::interval_CheckAddress(word inButtonStatus)
{

	unsigned char aValue;
	unsigned char aCV29;
	unsigned char aCV01;
	unsigned char aCV17;
	unsigned char aCV18;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_AddrChkMenu == 0)
			{
				gNo_AddrChkMenu = 1;
				
				//読み込み処理実行
				gDScmd.mParam1 = 29;
				gDScmd.mCmd = DSCMD_CVREAD;
				gDScmd.mWait = 1;
				
				//実行
				CVExeFunc(&aValue);
				
				if( aValue == 2)
				{
					/* Reset watchdog timer. */
					wdt_reset(); 
					
					aCV29 = CVValue;
					
					Serial.print("CV29=");
					Serial.println(aCV29);
					
					if( (aCV29 & 0b100000) > 0  )
					{
						
						//読み込み処理実行
						gDScmd.mParam1 = 17;
						gDScmd.mCmd = DSCMD_CVREAD;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue);
						
						if( aValue == 2)
						{
							aCV17 = CVValue;
							
							Serial.print("CV17=");
							Serial.println(aCV17);
							
							//読み込み処理実行
							gDScmd.mParam1 = 18;
							gDScmd.mCmd = DSCMD_CVREAD;
							gDScmd.mWait = 1;
							
							//実行
							CVExeFunc(&aValue);
							
							if( aValue == 2)
							{
								aCV18 = CVValue;
								
								Serial.print("CV18=");
								Serial.println(aCV18);
								
								//アドレス計算
								Address = aCV18 + ((aCV17 & 0b00111111) << 8);
								
								gNo_AddrChkMenu = 2;
								
							}
							else
							{
								gNo_AddrChkMenu = 99;
							}
							
							
						}
						else
						{
							gNo_AddrChkMenu = 99;
						}
						
					}
					else
					{
						//読み込み処理実行
						gDScmd.mParam1 = 1;
						gDScmd.mCmd = DSCMD_CVREAD;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue);
						
						if( aValue == 2)
						{
							aCV01 = CVValue;
							
							//アドレス計算
							Address = aCV01;
							
							gNo_AddrChkMenu = 2;
						}
						else
						{
							gNo_AddrChkMenu = 99;
						}
						
					}
					
					
				}
				else
				{
					gNo_AddrChkMenu = 99;
					
				}
				
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_AddrChkMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_AddrChkMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{

		}		
	}	
	
}

void DSSequence::interval_WriteAddress(word inButtonStatus)
{

	unsigned char aValue;
	unsigned char aOk = 0;
	unsigned char aCV29;
	unsigned char aCV01;
	unsigned char aCV17;
	unsigned char aCV18;
	
	
	//Check button changes
	if( inButtonStatus != gButtonBuf)
	{
		
		if( checkButton(gButtonBuf, inButtonStatus, BTN_ENT) == true )
		{
			if( gNo_AddrWriteMenu == 0)
			{
				//アドレス自動設定
				if( Address > 100)
				{
					aCV17 = 0b11000000 | (Address >> 8);
					aCV18 = 0xFF & Address;
					aCV01 = 0;
				}
				else
				{
					aCV01 = Address;
				}
				
				
				//表示切替
				gNo_AddrWriteMenu = 1;
				
				//読み込み処理実行
				gDScmd.mParam1 = 29;
				gDScmd.mCmd = DSCMD_CVREAD;
				gDScmd.mWait = 1;
				
				//実行
				CVExeFunc(&aValue);
				
				if( aValue == 2)
				{
					/* Reset watchdog timer. */
					wdt_reset(); 
					
					//表示切替
					gNo_AddrWriteMenu = 2;
					
					if( (aCV01 != 0) && ( (CVValue & 0b100000) != 0))
					{
						//SHORT Settings
						aCV29 = CVValue & 0b11011111;
						
						//CV29書込実行
						gDScmd.mParam1 = 29;
						gDScmd.mParam2 = aCV29;
						gDScmd.mCmd = DSCMD_CVWRITE;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue);
						
						if( aValue == 3)
						{
							aOk |= 0b0001;
						}
					}
					else if( (aCV01 == 0) && ( (CVValue & 0b100000) == 0))
					{
						//LONG Settings
						aCV29 = CVValue | 0b100000;
						
						//CV29書込実行
						gDScmd.mParam1 = 29;
						gDScmd.mParam2 = aCV29;
						gDScmd.mCmd = DSCMD_CVWRITE;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue);
						
						if( aValue == 3)
						{
							aOk |= 0b0001;
						}
						
					}
					else
					{
						aOk |= 0b0001;
					}
					
					Serial.print("CV29=");
					Serial.println(aCV29);
					
					if( aCV01 > 0)
					{
						//Short
						
						//CV01書込実行
						gDScmd.mParam1 = 1;
						gDScmd.mParam2 = aCV01;
						gDScmd.mCmd = DSCMD_CVWRITE;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue);
						
						if( aValue == 3)
						{
							aOk |= 0b1110;
						}
						
					}
					else
					{
						//Long
						
						//CV17書込実行
						gDScmd.mParam1 = 17;
						gDScmd.mParam2 = aCV17;
						gDScmd.mCmd = DSCMD_CVWRITE;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue);
						
						if( aValue == 3)
						{
							aOk |= 0b0110;
						}
						
						//CV18書込実行
						gDScmd.mParam1 = 18;
						gDScmd.mParam2 = aCV18;
						gDScmd.mCmd = DSCMD_CVWRITE;
						gDScmd.mWait = 1;
						
						//実行
						CVExeFunc(&aValue);
						
						if( aValue == 3)
						{
							aOk |= 0b1000;
						}
						
					}
					
					if( aOk == 0b1111)
					{
						gNo_AddrWriteMenu = 3;
						
					}
					else
					{
						gNo_AddrWriteMenu = 4;
						
					}
					
					
				}
				else
				{
					//CV29 エラー
					gNo_AddrWriteMenu = 99;
					
				}
				
				//表示処理
				gDisplayFlag |= 1;
			}
			else
			{
				//メニューに戻る
				gMode = MODE_MAINMENU;
				gNo_AddrWriteMenu = 0;
				
				//表示処理
				gDisplayFlag |= 1;
			}
			
		}
		else if( checkButton(gButtonBuf, inButtonStatus, BTN_ESC) == true )
		{
			//メニューに戻る
			gMode = MODE_MAINMENU;
			gNo_AddrWriteMenu = 0;
			
			//表示処理
			gDisplayFlag |= 1;
			
		}
		else
		{
			if( gNo_AddrWriteMenu == 0)
			{
				if( editNums(inButtonStatus, &Address, 9999) == true)
				{
					//表示処理
					gDisplayFlag |= 1;
				}
			}
		}		
	}	
	
}

bool DSSequence::checkButton(word inBuf, word inCurrent, word inButtonBit)
{

	if( ((inBuf & inButtonBit) == 0) && ((inCurrent & inButtonBit) > 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

bool DSSequence::checkButtonLeaved(word inBuf, word inCurrent, word inButtonBit)
{

	if( ((inBuf & inButtonBit) > 0) && ((inCurrent & inButtonBit) == 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

bool DSSequence::checkButtonContinuous(word inBuf, word inCurrent, word inButtonBit)
{

	if( ((inBuf & inButtonBit) > 0) && ((inCurrent & inButtonBit) > 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

void DSSequence::StrCpyFromROM(char *outpText, const char *inpROMText, byte inLen)
{
	byte i;
	
	for( i = 0; i < inLen; i++)
	{
		outpText[i] = (char)pgm_read_byte(inpROMText + i);
	}
	
	for( i = inLen; i < 16; i++)
	{
		outpText[i] = 0x00;
	}
}


word DSSequence::ProcessCheck( byte inTempA, byte inTempB, byte inTempC)
{
	byte aChanged = 0;
	word aButtonData = 0;
	
	switch(inTempA)
	{
	case 1:
		//SW9, 5
		aButtonData |= BTN_NUM5;
		break;
	case 2:
		//SW2, 1
		aButtonData |= BTN_NUM1;
		break;
	case 4:
		//SW13, 9
		aButtonData |= BTN_NUM9;
		break;
	case 8:
		//SW5, 4
		aButtonData |= BTN_NUM4;
		break;
	case 16:
		//SW14, CLR
		aButtonData |= BTN_CLR;
		break;
	}
		
	switch(inTempB)
	{
	case 1:
		//SW1, 0
		aButtonData |= BTN_NUM0;
		break;
	case 2:
		//SW3, 2
		aButtonData |= BTN_NUM2;
		break;
	case 4:
		//SW12, 8
		aButtonData |= BTN_NUM8;
		break;
	case 8:
		//SW6, ESC
		aButtonData |= BTN_ESC;
		break;
	case 16:
		//SW15, UP
		aButtonData |= BTN_UP;
		break;
	}

		
	switch(inTempC)
	{
	case 1:
		//SW10, 6
		aButtonData |= BTN_NUM6;
		break;
	case 2:
		//SW4, 3
		aButtonData |= BTN_NUM3;
		break;
	case 4:
		//SW11, 7
		aButtonData |= BTN_NUM7;
		break;
	case 8:
		//SW7, ENTER
		aButtonData |= BTN_ENT;
		break;
	case 16:
		//SW8, DOWN
		aButtonData |= BTN_DN;
		break;
	}

	return aButtonData;
	
}


byte DSSequence::GetButtonsFromAD(unsigned short inADData)
{
	/*
	R1	2.2kΩ
	R2	470Ω
	R3	1kΩ
	R4	2.2kΩ
	R5	6.8kΩ
	
	Threshold value:
	SW1: 82
	SW2: 290
	SW3: 525
	SW4: 747
	SW5: 942
	
	
	*/
	
	byte aRet = 0;
	
	if( inADData <= 82)
	{
		aRet = 1 << 0;
	}
	else if( inADData <= 290)
	{
		aRet = 1 << 1;
	}
	else if( inADData <= 525)
	{
		aRet = 1 << 2;
	}
	else if( inADData <= 747)
	{
		aRet = 1 << 3;
	}
	else if( inADData <= 942)
	{
		aRet = 1 << 4;
	}
	else
	{
		//Not pressed
	}
	
	
	return aRet;
}